/*
; MY-PRGM.
; ========

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:
;
; Project:      Kameleon376SBC
; Goal:		Demo of a C application.
;			This application shows how to launch processes and how to
;			protect critical resources (in this case the serial channel).
;
;			This code can be executed in two different ways:
;
;			1. Downloading mode:
;			   - Compile this application.
;			   - In the SerCom environment type "run sloader".
;			   - Send the binary by the serial link.
;
;			2. Romable mode:
;			   - Compile this application.
;			   - Catenate the binary at the end of the ROM modules.
;				 (after the "TERMINATOR.BIN" module).
;			   - In the SerCom environment type "run user".
;
; This module is (c) of Edoardo Franzi 1992-1998
; franzi@k-team.com
;
;------------------------------------------------------------------------
*/

#include	<kamsys.h>
#include 	<stdlib.h>
#include 	<stdio.h>

/*
 * Process 0
 * ^^^^^^^^^
 *
 * This process changes the state of the LED 0
 * every 500 ms.
 *
 */
 
void process_0()
	{

	for(;;)
		{
		tim_suspend_task(500);
		var_change_led(0);
		}
	}

/*
 * Process 1
 * ^^^^^^^^^
 *
 * This process tries to send a string on the "COM" channel
 * every 1000 ms.
 * The macro RESERVE_COM has to be used to reserve the
 * channel for this purpose.
 * If the "COM" channel is busy (used by an other process), the
 * control is given to the micro-kernel.
 *
 */

void process_1()
	{
	
	for(;;)
		{
		tim_suspend_task(1000);
		RESERVE_COM
		printf("K-System\n");
		RELEASE_COM
		}
	}

/*
 * Process 2
 * ^^^^^^^^^
 *
 * This process tries to send a string on the "COM" channel
 * every 500 ms.
 * The macro RESERVE_COM has to be used to reserve the
 * channel for this purpose.
 * If the "COM" channel is busy (used by an other process), the
 * control is given to the micro-kernel.
 *
 */

void process_2()
	{

	int	vCpt;

	vCpt = 0;
	for(;;)
		{
		tim_suspend_task(500);
		RESERVE_COM
		printf("%d, (c) Edoardo Franzi, K-Team S.A. 1997\n", vCpt);
		RELEASE_COM
		vCpt += 1;
		}
	}

/*
 * MAIN
 * ====
 *
 * - Initialise the used managers: tim & bios are initialised at the start-up.
 * - Launch all the processes.
 * - Kill the "main". At this moment only the launched processes are executed.
 *
 */

void main(void)
	{
	uint32	vIDProcess[3];
	int32	status;

	/*
	--------------------------|------------------------------------------------|
	*/

	static char	prName_0[] = "User 0 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_1[] = "User 1 process, EF-98                            Rev. 1.00\r\n";
	static char	prName_2[] = "User 2 process, EF-98                            Rev. 1.00\r\n";

	com_reset();
	var_reset();

	status = install_task(prName_0, 800, process_0);
	if (status == -1) exit(0);
	vIDProcess[0] = (uint32)status;

	status = install_task(prName_1, 800, process_1);
	if (status == -1) exit(0);
	vIDProcess[1] = (uint32)status;

	status = install_task(prName_2, 800, process_2);
	if (status == -1) exit(0);
	vIDProcess[2] = (uint32)status;
	}
