/**************************************************************************
Project name    :       sform.c
Purpose         :       Take a .code file and generates a S28 format file
Creation date   :       04/09/92
Author          :       F. Mondada
History.        :       F. Mondada, sun
                        F. Mondada, PC (linux) compatible...
                        F. Mondada, bug fixed in adress printing
***************************************************************************/

#include "gen_types.h"
#include <stdio.h>

#define L_PAQUET 100
#define L_ENTETE 256

enum { ARG_FILENAME = 1, ARG_COUNT };

/* ******************************************************************** */
/* ******************************************************************** */

char valtostr[16] = 
{'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};

void main(argc,argv)
  int32 argc;
  char *argv[];
  {
  FILE *f_in, *f_out;
  char  r_buffer[L_PAQUET], w_buffer[L_PAQUET*2];
  uint8 entete[L_ENTETE];
  char *r_ptr, *w_ptr, temp;
  int32 lr_size, lw_size, gw_size, gr_size = 0, check, i, check_gw, start;
  
  if ((argc != ARG_COUNT))
    {
    fprintf(stderr,"Usage: %s code_filename\n", argv[0]);
    exit(10);
    }
  if ((f_in = fopen(argv[ARG_FILENAME], "r")) == NULL)
    {
    fprintf(stderr, "File %s not found\n", argv[ARG_FILENAME]);
    exit(-1);
    }

  for (i=0; i< L_ENTETE; i++)
    {
    if( (fread(entete+i, 1, 1, f_in)) != 1)                  /* cut header! */
      {
      fprintf(stderr, ".code Header not readable\n");
      fclose(f_in);
      exit(-1);
      }
    }
  gw_size = (entete[32]* 0x1000000) + 
            (entete[33] * 0x10000) +
            (entete[34] * 0x100) +
            (entete[35]);
  start = gw_size;
            
  printf("S0030000FC\n");                                       /* header */

  do
    {
    lr_size = fread(r_buffer, 1, L_PAQUET, f_in);
    gr_size += lr_size;
    check_gw = gw_size;                              /* &0xFF -> first byte */


    for(check=lr_size+4+check_gw+(check_gw>>8)+(check_gw>>16), 
        i=0, r_ptr=r_buffer, w_ptr=w_buffer; i<lr_size; i++)
      {
      temp = *r_ptr;
      check = (check + temp)&0xFF;
      *w_ptr++ = valtostr[(temp>>4)&0x0F];
      *w_ptr++ = valtostr[(*r_ptr++) & 0x0F];
      }
    if (lr_size!=0)
      {
      printf("S2%02X%06X",lr_size+4,gw_size);
      lw_size = fwrite(w_buffer, 1, 2*lr_size, stdout)/2;       /* data */
      gw_size += lw_size;
      printf("%02X\n",(~check)&0xFF);
      }
    }
  while(!feof(f_in));
  
  printf("S804");                               /* end */
  printf("%06X", start);

  check=4+start+( start>>8)+(start>>16);
  printf("%02X\n",(~check)&0xFF);

  fclose(f_in);

  if (gr_size != (gw_size-start))
    {
    fprintf(stderr, "Error! readed size: %d, writed size: %d\n"
            ,gr_size,(gw_size-start));
    exit(-1);
    }
  exit(0);
 }



