/*
; BIOS and COM system call and definitions.
; =========================================

;------------------------------------------------------------------------
; Author:	Franzi Edo.	The 03/04/98
; Modifs:
;
; Project:	Kameleon376SBC
; Goal:		BIOS and COM system call and definitions.
;
; This module is (c) of Edoardo Franzi 1992-1998
; franzi@k-team.com
;
;------------------------------------------------------------------------
*/

#ifndef	_K376SBCASM_H
#define	_K376SBCASM_H

#include <ktypes.h>

#define	ABIGPTR	0x400					/* location of a big ptr	*/

/* Important structures */
/* ==================== */

typedef	struct	procDesc		PROCDESC;
typedef	struct	process			PROCESS;
typedef	struct	headerModule	HEADERMODULE;
typedef	struct	bigPtrTable		BIGPTRTABLE;

/* Structure of the process descriptor of the microkernel */
/* ------------------------------------------------------ */

struct	procDesc
	{
	uint32		oIDProc;				/* process ID						*/
	uint8		oStaProc;				/* process state					*/
	#define		BPRES	 	0			/* process present					*/
	#define		BLOCK		1			/* process lock						*/

	uint8		oReserve;				/* reserve							*/
	uint32		oMskEvent;				/* event mask						*/
	uint32		oMskSync;				/* synchro mask						*/
	PROCDESC	*oPtrCurrent;			/* ptr on the belong descriptor		*/
	PROCDESC	*oPtrBack;				/* ptr on the back descriptor		*/
	PROCDESC	*oPtrForward;			/* ptr on the forward descriptor 	*/
	char		*oPtrTextID;			/* ptr on the descriptor ID text	*/
	uint32		oTStopProc;				/* a wait time						*/
	uint32		*oPtrStkPro;			/* process stack					*/
	};

/* Structure of the process descriptor (pointed by (oDescProc,Ax)) */
/* --------------------------------------------------------------- */

struct	process
	{
	uint32		*oPtrProcCode;			/* ptr on the process code				*/
	uint32		*oPtrProcStack;			/* ptr on the process stack				*/
	char		*oPtrProcText;			/* ptr on the process text ID string	*/
	};

/* Structure of the header module */
/* ------------------------------ */

struct	headerModule
	{
	uint32		oBRACode;				/* jump to the module												  	*/
	uint32		oIDModule;				/* ID of the module T T n n											  	*/
	#define		KIDFUNCTION	'FU'		/*					F U x x ---> stand alone executable function code 	*/
	#define		KIDPROTOCOL	'PR'		/*					P R x x ---> SerCom protocol function code		  	*/
	#define		KIDTOOL		'TO'		/*					T O x x ---> SerCom tool function code			  	*/
	#define		KIDBIOS		'BI'		/*					B I x x ---> BIOS								  	*/
	#define		KIDSTARTUP	'ST'		/*					S T x x ---> Start-up code						  	*/
	#define		KIDLIB		'LI'		/*					L I x x ---> LIB								  	*/
	#define		KIDDEBUG	'DG'		/*					D G x x ---> Debug								  	*/
	#define		KIDPROCESS	'TA'		/*					T A x x ---> Proc								  	*/

	uint32		*oPtrCODE;				/* ptr on the CODE segment			*/
	uint32		oLnCODE;				/* lenght of the CODE segment		*/
	uint32		*oPtrDATA;				/* ptr on the DATA segment			*/
	uint32		oLnDATA;				/* lenght of the DATA segment		*/
	uint32		*oPtrStrAppl;			/* ptr on the string				*/
	uint32		oRevision;				/* revision							*/
	uint32		*oPtrStrHelp;			/* ptr on the help string			*/
	uint32		oFlType;				/* type of module					*/
	#define		BSHOW			0		/* visible in help					*/
	#define		BEXECSERCOM		1		/* executable from SerCom			*/
	#define		BEXECRUN		2		/* executable from run				*/
	#define		BCONFIDENTIAL	3		/* no comment						*/

	PROCESS		*oDescProc;				/* ptr on the process descriptor	*/
	uint32		oRes2;					/* reserve 2						*/
	uint32		oRes1;					/* reserve 1						*/
	};

/* Structure of the main big ptr table */
/* ----------------------------------- */

struct	bigPtrTable
	{
	HEADERMODULE	*oPtrFrstMod;		/* ptr on a first module			*/
	uint8			*oPtrTurOTable;		/* ptr on a turret order table		*/
	uint8			*oPtrIOChan;		/* ptr on a I/O channel device		*/
	uint8			*oPtrIDComm;		/* ptr on a ID communication turret	*/
	uint8			*oPtrStUPSys;		/* ptr on a flash start-up system	*/
	uint8			*oPtrStUPUsr;		/* ptr on a flash start-up user		*/
	uint8			*oPtrFrstFreeMe;	/* ptr on a first free memory		*/
	uint8			*oPtrDownLCode;		/* ptr on a downloaded code			*/
	};

/* Special macros */
/* ============== */

#define	RESERVE_MSG(channelNb)	while (msg_reserve_channel(channelNb) == -1) tim_switch_fast();
#define	RELEASE_MSG(channelNb)	msg_release_channel(channelNb);

#define	RESERVE_SER				while (ser_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_SER				ser_release_channel();

#define	RESERVE_USART			while (usart_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_USART			usart_release_channel();

#define	RESERVE_MMA0			while (mma_reserve_channel_0() == -1) tim_switch_fast();
#define	RELEASE_MMA0			mma_release_channel_0();
#define	RESERVE_MMA1			while (mma_reserve_channel_1() == -1) tim_switch_fast();
#define	RELEASE_MMA1			mma_release_channel_1();
#define	RESERVE_MMA2			while (mma_reserve_channel_2() == -1) tim_switch_fast();
#define	RELEASE_MMA2			mma_release_channel_2();

#define	RESERVE_COM				while (com_reserve_channel() == -1) tim_switch_fast();
#define	RELEASE_COM				com_release_channel();

#define	DEBUGGER				asm("	trap	#7");

#define	INTERRUPTION_SET		asm("	move.w	#$2400,SR");

#define	INTERRUPTION_OFF		asm("	move.w	SR,-(SP)");	\
								asm("	move.w	#$2700,SR");

#define	INTERRUPTION_RESTORED	asm("	move.w	(SP)+,SR");

/* BIOS system calls */
/* ================= */

#define	CALL_BIOS_0(callNumber)	asm("	trap	#0"); \
								asm("	dc.w	"#callNumber"");

#define	CALL_BIOS_1(callNumber)	asm("	move.l	8(sp),-(sp)"); \
								asm("	trap	#0"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#1*4,sp");

#define	CALL_BIOS_2(callNumber)	asm("	move.l	12(sp),-(sp)"); \
								asm("	move.l	12(sp),-(sp)"); \
								asm("	trap	#0"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#2*4,sp");

#define	CALL_BIOS_3(callNumber)	asm("	move.l	16(sp),-(sp)"); \
								asm("	move.l	16(sp),-(sp)"); \
								asm("	move.l	16(sp),-(sp)"); \
								asm("	trap	#0"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#3*4,sp");

#define	CALL_BIOS_4(callNumber)	asm("	move.l	20(sp),-(sp)"); \
								asm("	move.l	20(sp),-(sp)"); \
								asm("	move.l	20(sp),-(sp)"); \
								asm("	move.l	20(sp),-(sp)"); \
								asm("	trap	#0"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#4*4,sp");

#define	CALL_BIOS_5(callNumber)	asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	trap	#0"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#5*4,sp");

/* BIOS 0..49 */
/* ---------- */

extern	void		bios_reset(void);
extern	char		*bios_get_ident(void);
extern	uint32		bios_get_rev(void);
extern	uint32		bios_get_system(void);
extern	void		bios_restart_system(void);
extern	uint32		bios_get_node(void);
extern	int32		bios_restart_launch(uint32 idFunction, uint32 baudrate);

/* TIM 50..99 */
/* ---------- */

extern	void		tim_reset(void);
extern	int32		tim_new_inst_task(char *textId, uint32 *stack, void (*procedure)());
extern	int32		tim_remove_inst_task(uint32 id);
extern	void		tim_suspend_task(uint32 time);
extern	void		tim_generate_event(void);
extern	void		tim_wait_event(uint32 taskMask);
extern	uint32		tim_get_id(void);
extern	uint32		tim_get_ticcount(void);
extern	void		tim_switch_fast(void);
extern	void		tim_lock(void);
extern	void		tim_unlock(void);
extern	int32		tim_define_association(char *reference, uint32 *general);
extern	int32		tim_find_association(char *reference);
extern	int32		tim_remove_association(char *reference);
extern	void		tim_wait_sync(uint32 syncMask);
extern	PROCDESC	*tim_get_task_des_ptr(void);

/* SENS 100..149 */
/* ------------- */

extern	void		sens_reset(void);
extern	int32		sens_get_ana_value(uint32 inputNb);

/* MSG 150..199 */
/* ------------ */

extern	void		msg_reset(void);
extern	int32		msg_reserve_channel(uint32 channelNb);
extern	int32		msg_release_channel(uint32 channelNb);
extern	int32		msg_send_message(uint8 *mesgS, uint32 sizeS);
extern	int32		msg_receive_message(uint8 *mesgR, uint32 sizeR);
extern	int32		msg_snd_rec_message(uint8 *mesgS, uint32 sizeS, uint8 *mesgR, uint32 sizeR, uint32 rep);

/* VAR 200..249 */
/* ------------ */

extern	void		var_reset(void);
extern	uint32		var_get_jumper(void);
extern	int32		var_on_led(uint32 ledNb);
extern	int32		var_off_led(uint32 ledNb);
extern	int32		var_change_led(uint32 ledNb);
extern	void		var_set_irq_vector(void(*procedure)());
extern	void		var_enable_irq(void);
extern	void		var_disable_irq(void);
extern	int32		var_set_exception(void(*procedure)(), uint32 exceptionNb);
extern	int32		var_cpu_speed(uint32 CPUSpeedValue);
extern	uint32		var_get_extension(uint32 address);
extern	void		var_put_extension(uint32 address, uint32 binaryValue);

/* SER 250..299 */
/* ------------ */

extern	void		ser_reset(void);
extern	int32		ser_reserve_channel(void);
extern	void		ser_release_channel(void);
extern	int32		ser_config(uint32 baudrate);
extern	int32		ser_send_buffer(uint8 *buffer, uint32 size);
extern	int32		ser_receive_byte(void);
extern	int32		ser_tx_status(void);
extern	int32		ser_rx_status(void);

/* STR 300..349 */
/* ------------ */

extern	void		str_reset(void);
extern	int32		str_cnvt_dascii_bin32(uint32 *binary, char *ascii, uint32 paramNb);
extern	int32		str_cnvt_dascii_bin8(uint8 *binary, char *ascii, uint32 paramNb);
extern	void		str_cnvt_bin32_dascii(char *ascii, uint32 *binary, uint32 paramNb);
extern	void		str_cnvt_bin8_dascii(char *ascii, uint8 *binary, uint32 paramNb);
extern	uint32		str_get_size_ascii(char *ascii);
extern	int32		str_skip_param_protocol(char *ascii, uint32 paramNb);
extern	char		*str_cnvt_vbin_dascii(char *ascii, uint32 binaryValue);
extern	int32		str_skip_param(char *ascii, uint32 paramNb);
extern	char		*str_cnvt_vbin_hascii(char *ascii, uint32 binaryValue);
extern	int32		str_cnvt_hascii_bin32(uint32 *binary, char *ascii, uint32 paramNb);
extern	int32		str_cnvt_hascii_bin8(uint8 *binary, char *ascii, uint32 paramNb);
extern	void		str_cnvt_bin32_hascii(char *ascii, uint32 *binary, uint32 paramNb);
extern	void		str_cnvt_bin8_hascii(char *ascii, uint8 *binary, uint32 paramNb);

/* MMA 350..399 */
/* ------------ */

extern	void		mma_reset(void);
extern	int32		mma_reserve_channel_0(void);
extern	int32		mma_reserve_channel_1(void);
extern	int32		mma_reserve_channel_2(void);
extern	void		mma_release_channel_0(void);
extern	void		mma_release_channel_1(void);
extern	void		mma_release_channel_2(void);
extern	int32		mma_send_buffer_0(uint8 *buffer, uint32 size);
extern	int32		mma_send_buffer_1(uint8 *buffer, uint32 size);
extern	int32		mma_send_buffer_2(uint8 *buffer, uint32 size);
extern	int32		mma_receive_byte_0(void);
extern	int32		mma_receive_byte_1(void);
extern	int32		mma_receive_byte_2(void);
extern	int32		mma_tx_status_0(void);
extern	int32		mma_tx_status_1(void);
extern	int32		mma_tx_status_2(void);
extern	int32		mma_rx_status_0(void);
extern	int32		mma_rx_status_1(void);
extern	int32		mma_rx_status_2(void);

/* USART 400..449 */
/* -------------- */

extern	void		usart_reset(void);
extern	int32		usart_reserve_channel(void);
extern	void		usart_release_channel(void);
extern	int32		usart_config(uint32 baudrate);
extern	int32		usart_send_buffer(uint8 *buffer, uint32 size);
extern	int32		usart_receive_byte(void);
extern	int32		usart_tx_status(void);
extern	int32		usart_rx_status(void);

/* COM system calls */
/* ================ */

#define	CALL_COM_0(callNumber)	asm("	trap	#6"); \
								asm("	dc.w	"#callNumber"");

#define	CALL_COM_1(callNumber)	asm("	move.l	8(sp),-(sp)"); \
								asm("	trap	#6"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#1*4,sp");

#define	CALL_COM_2(callNumber)	asm("	move.l	12(sp),-(sp)"); \
								asm("	move.l	12(sp),-(sp)"); \
								asm("	trap	#6"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#2*4,sp");

#define	CALL_COM_3(callNumber)	asm("	move.l	16(sp),-(sp)"); \
								asm("	move.l	16(sp),-(sp)"); \
								asm("	move.l	16(sp),-(sp)"); \
								asm("	trap	#6"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#3*4,sp");

#define	CALL_COM_4(callNumber)	asm("	move.l	20(sp),-(sp)"); \
								asm("	move.l	20(sp),-(sp)"); \
								asm("	move.l	20(sp),-(sp)"); \
								asm("	move.l	20(sp),-(sp)"); \
								asm("	trap	#6"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#4*4,sp");

#define	CALL_COM_5(callNumber)	asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	move.l	24(sp),-(sp)"); \
								asm("	trap	#6"); \
								asm("	dc.w	"#callNumber""); \
								asm("	add.l	#5*4,sp");

/* COM 0..15 */
/* --------- */

extern	void		com_reset(void);
extern	int32		com_reserve_channel(void);
extern	void		com_release_channel(void);
extern	int32		com_send_buffer(uint8 *buffer, uint32 size);
extern	int32		com_receive_byte(void);
extern	int32		com_get_status_channel(void);
#endif
