/*
 *	MATH.H		Math function declarations
 */

#ifndef _MATH_H
#define	_MATH_H

extern	int	abs(int value);
extern	double	acos(double x);
extern	double	asin(double x);
extern	double	atan(double x);
extern	double	atan2(double y, double x); /* atan(y / x) */
extern	double	atof(const char *s);	/* ascii -> number */
extern	double	ceil(double x);		/* f <= i */
extern	double	cos(double x);
extern	double	cosh(double x);
extern	double	exp(double x);		/* e ** x */
extern	double	fabs(double x);
extern	double	floor(double x);	/* i <= f */
extern	double	fmod(double x, double y); /* x modulo y */
extern	double	frexp(double x, int *exp); /* res * (2 ** exp) = x */
extern	double	hypot(double x, double y); /* sqrt(x*x + y*y) */
extern	int	isinf(double x);	/* x = +-INF ? */
extern	int	isnan(double x);	/* x = +-NAN ? */
extern	long	labs(long value);
extern	double	ldexp(double x, int exp); /* x * (2 ** exp) */
extern	double	log(double x);		/* ln(x) */
extern	double	log10(double x);	/* log10(x) */
extern	double	modf(double x, double *ipart); /* ipart.res = x */
extern	double	pow(double x, double y); /* x ** y */
extern	double	pow10(int exp);		/* 10 ** exp */
extern	double	sin(double x);
extern	double	sinh(double x);
extern	double	sqrt(double x);
extern	double	tan(double x);
extern	double	tanh(double x);

/* Some useful constants */
#define M_E	2.71828182845904523536	/* base e */
#define M_LOG2E	1.44269504088896340736	/* log2(e) */
#define M_LOG10E 0.434294481903251827651 /* log10(e) */
#define M_LN2	0.693147180559945309417	/* ln(2) */
#define M_LN10	2.30258509299404568402	/* ln(10) */
#define M_PI	3.14159265358979323846	/* pi */
#define M_PI_2	1.57079632679489661923	/* pi/2 */
#define M_PI_4	0.785398163397448309116	/* pi/4 */
#define M_1_PI	0.318309886183790671538	/* 1/pi */
#define M_2_PI	0.636619772367581343076	/* 2/pi */
#define M_2_SQRTPI 1.12837916709551257390 /* 2/sqrt(pi) */
#define M_SQRT2	1.41421356237309504880	/* sqrt(2) */
#define M_SQRT1_2 0.707106781186547524401 /* sqrt(0.5) */

#define HUGE_VAL DBL_MAX

#endif	/* _MATH_H */

